/*!\file
 * \brief    ,            .
 *
 * Copyright(C)   , 2017
 * Copyright(C)     
 *   .
 *
 *  ,    ,    
 *  ,       
 *  " ".     " " 
 *  - , " "    , 
 *   ,      , 
 *         . 
 *  " "       
 *  ,       ,  , 
 *      ,  - . 
 *  " "         
 *   .  " "     
 *   ,        
 *     .
 *
 *   , ,      
 *       " "  
 *       ,  , 
 *         
 * " ".         .
 */

#ifndef TMTX_H_
#define TMTX_H_

#include "types.h"

/** \brief       
 */
typedef struct
{
    volatile uint32_t DATA_PTR;                                                 ///<   
    volatile uint32_t HEADER;                                                   ///< 
}TCRX_DSC_T;

/** \brief          
 */
typedef struct
{
    __RW uint32_t DSCR1_PTR;                                                    /**< : 0x000  1:     */
    __RW uint32_t DSCR1_FLAG_SIZE;                                              /**< : 0x004  1:      */
    __RW uint32_t DSCR2_PTR;                                                    /**< : 0x008  2:     */
    __RW uint32_t DSCR2_FLAG_SIZE;                                              /**< : 0x00C  2:      */
    __RW uint32_t GLOBAL_ENABLE;                                                /**< : 0x010      */
    __RW uint32_t CODER_SEL;                                                    /**< : 0x014   */
    __RW uint32_t PHY_FIFO_UNDERRUN;                                            /**< : 0x018   FIFO */
    __RW uint32_t MARKER_AND_FRAME_SIZE;                                        /**< : 0x01C      */
    __RW uint32_t MARKER1;                                                      /**< : 0x020   1 */
    __RW uint32_t MARKER2;                                                      /**< : 0x024   2 */
    __RW uint32_t MARKER3;                                                      /**< : 0x028   3 */
    __RW uint32_t MARKER4;                                                      /**< : 0x02C   4 */
    __RW uint32_t MARKER5;                                                      /**< : 0x030   5 */
    __RW uint32_t MARKER6;                                                      /**< : 0x034   6 */
    __RW uint32_t CODERS_CONFIG;	                                        /**< : 0x038   */
}TMTX_T; 
 
/**
 * @brief   DSCR1_PTR, DSCR2_PTR
 */    
#define TMTX_DSCR_PTR_DSCR_PTR(x)       ((uint32_t)((x & 0x3FFFFFFF) << 2))     /**<         .
                                                                                          4  ,  2    */


/**
 * @brief   DSCR1_SIZE_FLAG, DSCR2_SIZE_FLAG
 */    
#define TMTX_DSCR1_SIZE_FLAG_DATA_SIZE(x)            ((uint32_t)((x & 0x7FF) << 0))  /**<        (0x000 = 1 ) */
#define TMTX_DSCR1_SIZE_FLAG_DATA_PRES               ((uint32_t)(1 << 31))           /**<     */

/**
 * @brief   GLOBAL_ENABLE
 */
#define TMTX_GLOBAL_ENABLE                      ((uint32_t)(1 << 0))            /**<   ,     */
   
/**
 * @brief   PHY_FIFO_UNDERRUN
 */
#define TMTX_PHY_FIFO_UNDERRUN                  ((uint32_t)(1 << 0))            /**<    FIFO */

/**
 * @brief     CODER_SEL
 */
typedef enum 
{
    TMTX_CODER_SEL_LDPC   = ((uint32_t)(1 << 0)),                               /**<  LDPC- */
    TMTX_CODER_SEL_TURBO  = ((uint32_t)(2 << 0)),                               /**<  Turbo- */
    TMTX_CODER_SEL_RS     = ((uint32_t)(3 << 0)),                               /**<  RS- */
    TMTX_CODER_SEL_NONE   = ((uint32_t)(0 << 0)),                               /**<   */
    TMTX_CODER_CONV_SEL    = ((uint32_t)(1 << 2)),                              /**<    */
    TMTX_CODER_RAND_SEL    = ((uint32_t)(1 << 3)),                              /**<   */
}TMTX_CODER_SEL_T;

/**
 * @brief   MARKER_AND_FRAME_SIZE
 */
#define TMTX_MARKER_AND_FRAME_FRAME_SIZE(x)     ((uint32_t)((x & 0xFFFF) << 0)) /**<      */
#define TMTX_MARKER_AND_FRAME_MARKER_SIZE(x)    ((uint32_t)((x & 0xFF) << 16))  /**<      */

/**
 * @brief     CODERS_CONFIG
 */
typedef enum 
{
    TMTX_CC_CONV_1DIV2   = ((uint32_t)(1 << 0)),                                /**<    1/2 */
    TMTX_CC_CONV_2DIV3   = ((uint32_t)(2 << 0)),                                /**<    2/3 */
    TMTX_CC_CONV_3DIV4   = ((uint32_t)(3 << 0)),                                /**<    3/4 */
    TMTX_CC_CONV_5DIV6   = ((uint32_t)(5 << 0)),                                /**<    5/6 */
    TMTX_CC_CONV_7DIV8   = ((uint32_t)(7 << 0)),                                /**<    7/8 */
}TMTX_CODERS_CONFIG_CONV_T;

typedef enum 
{
    TMTX_CC_TURBO_1DIV2  = ((uint32_t)(0 << 8)),                                /**<  Turbo  1/2 */
    TMTX_CC_TURBO_1DIV3  = ((uint32_t)(1 << 8)),                                /**<  Turbo  1/3 */ 
    TMTX_CC_TURBO_1DIV4  = ((uint32_t)(2 << 8)),                                /**<  Turbo  1/4 */ 
    TMTX_CC_TURBO_1DIV6  = ((uint32_t)(3 << 8)),                                /**<  Turbo  1/6 */ 
}TMTX_CODERS_CONFIG_TURBO_T;                                                                                      

typedef enum 
{
    TMTX_CC_RS_TYPE_239     = ((uint32_t)(0 << 16)),                            /**<  RS   (255,239)*/
    TMTX_CC_RS_TYPE_223     = ((uint32_t)(1 << 16)),                            /**<  RS   (255,223)*/ 
    TMTX_CC_RS_TYPE_239_CONV     = ((uint32_t)(2 << 16)),                       /**<  RS  dual basis converted (255,239)*/ 
    TMTX_CC_RS_TYPE_223_CONV     = ((uint32_t)(3 << 16)),                       /**<  RS  dual basis converted (255,223)*/ 
}TMTX_CODERS_CONFIG_RS_T;

typedef enum 
{
	TMTX_CC_LDPC_TYPE_7136   = ((uint32_t)(0 << 24)),                       /**<  LDPC  (8160 ,7136) */  
}TMTX_CODERS_CONFIG_LDPC_T;




/**
 * @brief ,     
 */
#define TMTX_FRAME_SIZE_MAX                     2048                            /**<          (  ) */ 
#define TMTX_FRAME_SIZE_LDPC                    892                             /**<      LDPC */ 
#define TMTX_FRAME_SIZE_TURBO                   223                             /**<      TURBO */
#define TMTX_FRAME_SIZE_RS_239                  239                             /**<      - 239 */
#define TMTX_FRAME_SIZE_RS_223                  223                             /**<      - 223 */

#define TMTX_BIT_OUT_SIZE_MAX                   (TMTX_FRAME_SIZE_MAX*8)         /**<           (  ) */ 
#define TMTX_BIT_OUT_SIZE_LDPC_7136             8160                            /**<      LDPC  7136 */
#define TMTX_BIT_OUT_SIZE_LDPC_7156             8176                            /**<      LDPC  7156 */
#define TMTX_BIT_OUT_SIZE_TURBO_1DIV2           3576                            /**< (223*8*2+4*2)      TURBO    1/2 */
#define TMTX_BIT_OUT_SIZE_TURBO_1DIV3           5364                            /**< (223*8*3+4*3)      TURBO    1/3 */
#define TMTX_BIT_OUT_SIZE_TURBO_1DIV4           7152                            /**< (223*8*4+4*4)      TURBO    1/4 */
#define TMTX_BIT_OUT_SIZE_TURBO_1DIV6           10728                           /**< (223*8*6+4*6)      TURBO    1/6 */
#define TMTX_BIT_OUT_SIZE_RS                    2040                            /**< (255*8)       - (   ) */

#define TMTX_MAX_MARKER_SIZE			192				/**<      */
#define TMTX_MAX_DSCR_SIZE                      2048                            /**<      */

/**
 * @brief        1
 * @param addr:  ,      
 */
void TMTX_Set_DSCR1_Pointer(uint32_t addr);

/**
 * @brief        2
 * @param addr:  ,      
 */
void TMTX_Set_DSCR2_Pointer(uint32_t addr);

/**
 * @brief    1
 * @param size:    
 */
void TMTX_Set_DSCR1_Size(uint32_t size);

/**
 * @brief    2
 * @param size:    
 */
void TMTX_Set_DSCR2_Size(uint32_t size);

/**
 * @brief     ,     1   
 */
void TMTX_DSCR1_Data_Ready();

/**
 * @brief     ,     2   
 */
void TMTX_DSCR2_Data_Ready();

/**
 * @brief   Global Enable :   ,    
 */
void TMTX_Global_Enable();

/**
 * @brief   Global Enable :  , fifo, ahb      
 */
void TMTX_Global_Disable();

/**
 * @brief    
 * @return   ,   :
 *                                              - TMTX_CODER_SEL_LDPC
 *                                              - TMTX_CODER_SEL_TURBO 
 *                                              - TMTX_CODER_SEL_RS 
 *                                              - TMTX_CODER_SEL_NONE 
 */
uint32_t TMTX_Get_Coder();

/**
 * @brief       FIFO
 * @return  , SET or RESET
 */
FlagStatus TMTX_FIFO_Underrun();

/**
 * @brief     FIFO
 */
void TMTX_FIFO_Underrun_Clear_Flag();

/**
 * @brief       
 * @param marker_size:   ( )
 * @param frame_size:    ( )
 */
void TMTX_Set_Marker_And_Frame_Size(uint8_t marker_size, uint32_t frame_size);

/**
 * @brief     
 * @return    ,   :
 *                                              - TMTX_CC_CONV_1DIV2
 *                                              - TMTX_CC_CONV_2DIV3
 *                                              - TMTX_CC_CONV_3DIV4 
 *                                              - TMTX_CC_CONV_5DIV6
 *                                              - TMTX_CC_CONV_7DIV8
 */
uint32_t TMTX_Get_Conv_Type();

/**
 * @brief     
 * @return    ,   :
 *                                              - TMTX_CC_TURBO_1DIV2
 *                                              - TMTX_CC_TURBO_1DIV3
 *                                              - TMTX_CC_TURBO_1DIV4 
 *                                              - TMTX_CC_TURBO_1DIV6
 */
uint32_t TMTX_Get_Turbo_Type();
	
/**
 * @brief     -
 * @return    -,   :
 *                                              - TMTX_CC_RS_TYPE_239
 *                                              - TMTX_CC_RS_TYPE_223
 *                                              - TMTX_CC_RS_TYPE_239_CONV 
 *                                              - TMTX_CC_RS_TYPE_223_CONV
 */
uint32_t TMTX_Get_RS_Type();


/**
 * @brief  ,     LDPC
 * @return    LDPC,   :
 *                                              - TMTX_CC_LDPC_TYPE_7136
 *                                              - TMTX_CC_LDPC_TYPE_7156
 */
uint32_t TMTX_Get_LDPC_Type();

/**
 * @brief ,   ,   ,  
 */
uint32_t TMTX_GetFrameSize();

/**
 * @brief   .      .
 * @param marker_size:    .    192
 * @param marker1:  0 - 31 .      marker1
 * @param marker2:  32 - 63 .      marker_size > 32
 * @param marker3:  64 - 95 .      marker_size > 64
 * @param marker4:  96 - 127 .      marker_size > 96
 * @param marker5:  128 - 159 .      marker_size > 128
 * @param marker6:  160 - 191 .      marker_size > 160
 */
void TMTX_SetSyncMarker(uint8_t marker_size, 
                        uint32_t marker1, 
                        uint32_t marker2, 
                        uint32_t marker3, 
                        uint32_t marker4, 
                        uint32_t marker5, 
                        uint32_t marker6);

/**
 * @brief        ,     .
 * @param coder_sel:    (LDPC, Turbo, RS)
 * @param coder_config:      
 * @param rand_state: / 
 * @param conv_state: /  
 * @param conv_speed:     
 */
void TMTX_SetEncoder(uint32_t coder_sel, 
                     uint32_t coder_config, 
                     uint32_t rand_state, 
                     uint32_t conv_state, 
                     uint32_t conv_speed);


#endif